package ExSite::Captcha;
#-----------------------------------------------------------------------
#
#   Copyright 2001-2007 Exware Solutions, Inc.  http://www.exware.com
#
#   This file is part of ExSite WebWare (ExSite, for short).
#
#   ExSite is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   ExSite is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with ExSite; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   Users requiring warranty coverage and/or support may arrange alternate
#   commercial licensing for ExSite, by contacting Exware Solutions 
#   via the website noted above.
#
#----------------------------------------------------------------------------

=pod

=head1 ExSite::Captcha

The captcha tool generates and processes "captchas" (Completely
Automated Public Turing test to tell Computers and Humans Apart).  Two
types of captchas are supported:

=over 4

=item * image catpchas

These use distorted images of text, requiring the user to transcribe the
text into plain-text form.  Some image captchas have been successfully
attacked programatically, but the ExSite captchas should still be sufficient
to filter out most nuisance bots.

=item * text captchas

These use plain-text riddles, requiring the user to understand a question 
and provide a precise answer.  These captchas are relatively easy to defeat,
so we provide an extra degree of difficulty by randomly alternating between
several styles of question/answer, and relying in some cases on a dictionary
of tens of thousands of words as a source of challenge data.  Text captchas 
are useful as an accessible fallback for visually-impaired readers, or for 
browsers that do not support images.

=back

=head1 Usage

Usage is best illustrated with a simple example.  This CGI program
generates and processes a simple form containing only a captcha.

    #/usr/bin/perl
    use strict;
    use ExSite::Config;
    use ExSite::Input;
    use ExSite::Captcha;
    
    &exsite_init;
    print "content-type:text/html\n\n";
    
    my $input = ExSite::Input->new()->combine();
    my $c = new ExSite::Captcha(textmode=>$input->{captchamode});

    # evaluate the user's response
    if ($input->{captcha}) {
        if ($c->pass()) {
            print "Thanks, that's the correct answer!";
    	    exit;
        }
        else {
    	    print "Sorry, that's not the correct answer.";
        }
    }

    # output a new captcha
    my $captcha = $c->make();
    
    print <<END;
    <form method=POST>
    $captcha<br>
    <input type=submit>
    </form>
    END

=head2 C<new()>

B<Usage:> C<my $captcha = new ExSite::Captcha(%opt);>

The C<new()> method accepts a number of optional parameters:

=over 4

=item textmode

Set to 1/true to use a text catpcha.  Set to the value of the input
parameter "captchamode" to have your captcha automatically switch to
text mode on user request.

=item answer_field

The name of the form field containing the user's guess.
Defaults to "captcha".

=item solution_field

The name of the hidden form field containing the encrypted solution.
Defaults to "captcha_solution".

=item image_id

The ID of the SPAN element wrapping the captcha image.
Defaults to "captcha_image".

=item prompt

The text string prompting the user's input on image captchas.
Defaults to "Enter the text you see:".

=back

=head2 C<make($key)>

B<Usage:> C<print $captcha->make();>

C<make()> accepts an optional parameter C<$key>, which is one of:

=over 4

=item * the text string to encode in the image (max 5 chars)

=item * the text captcha algorithm to use (1-4)

=back

=head2 C<pass($guess,$encoded)>

B<Usage:> C<if ($captcha->pass()) { ...>

C<pass()> returns a non-null value if the user's guess is correct, and
nothing if not.  You can pass the user's guess and the encoded answer as
parameters, but it will try to deduce these from the web inputs if not 
provided.

=head1 Requirements

The CGI program C<captcha.cgi> generates the catpcha image in real-time.
It uses the C<convert> program (part of the ImageMagick suite) to write
the captcha text on top of a blank background (_ExSite/images/captcha.png).
By default the C<ps:Courier-Bold> font is used.  If not installed, another
font will need to be specified.

Text captchas use the system dictionary /usr/share/dict/words to pull
random word data out for some of the riddles.

=cut

use strict;
use ExSite::Config;
use ExSite::Base;
use ExSite::Misc;
use ExSite::Crypt;

use vars qw(@ISA);
@ISA = qw(ExSite::Base);

use vars qw(@RANK);

# some human-readable position words
@RANK = ( "first", "second", "third", "fourth", "fifth", 
	 "sixth", "seventh", "eighth", "ninth", "tenth");

sub make {
    my ($this,$key) = @_;
    if ($this->get("textmode")) {
	return $this->captcha_text($key);
    }
    else {
	return $this->captcha_image($key);
    }
}

sub pass {
    my ($this,$guess,$encoded_key) = @_;
    my $idata = ExSite::Input->new()->combine();
    # $guess is what the user/robot answered
    if (! $guess) {
	# see if we can find it in the form data
	my $field = $this->get("answer_field") || "captcha";
	$guess = $idata->{$field};
	return 0 if (! $guess);
    }
    # $encoded_key is the solution, encrypted
    if (! $encoded_key) {
	# see if we can find it in the form data
	my $solution = $this->get("solution_field") || "captcha_solution";
	$encoded_key = $idata->{$solution};
	return 0 if (! $encoded_key);
    }
    my $c = new ExSite::Crypt;
    my $decoded_key = $c->decrypt($encoded_key);
    return $guess =~ /^$decoded_key$/i;   # allow any case
}

sub captcha_image {
    my ($this,$key) = @_;
    if (! $key) {
	$key = 
	    $this->get("key") ||
	    &randtext(5,"abcdefghijkmnpqrstuvwxwz23456789");
    }
    my $c = new ExSite::Crypt;
    my $key_crypt = $c->encrypt($key);
    
    my $img_id = $this->get("image_id") || "captcha_image";
    my $solution = $this->get("solution_field") || "captcha_solution";
    my $answer = $this->get("answer_field") || "captcha";
    my $prompt = $this->get("prompt") || "Enter the text you see:";
    my $link = &relink(captchamode=>"text");
    
    return "<div class=\"captcha\">
<table cellspacing=\"0\" cellpadding=\"0\"><tr><td>
<span id=\"$img_id\">
<img src=\"$config{server}{CGIpath}/$config{prog}{captcha}?$key_crypt\" height=\"40\" width=\"100\">
<input type=\"hidden\" name=\"$solution\" value=\"$key_crypt\">
</span>
</td><td>
&nbsp;$prompt <input type=text name=\"$answer\" size=\"8\">
</td></tr></table>
<a href=\"$link\">Can't see the image?</a>
</div>\n";
}

sub captcha_text {
    my ($this,$n) = @_;
    my ($challenge,$response);
    if (! $n) { $n = int(rand(4))+1; }
    if ($n == 1)    { ($challenge,$response) = $this->captcha_text1; }
    elsif ($n == 2) { ($challenge,$response) = $this->captcha_text2; }
    elsif ($n == 3) { ($challenge,$response) = $this->captcha_text3; }
    else            { ($challenge,$response) = $this->captcha_text4; }
    
    my $c = new ExSite::Crypt;
    my $key_crypt = $c->encrypt($response);

    my $solution = $this->get("solution_field") || "captcha_solution";
    my $answer = $this->get("answer_field") || "captcha";
    
    return "<div class=\"captcha\">
<input type=\"hidden\" name=\"$solution\" value=\"$key_crypt\">
$challenge<br>
<input type=text name=\"$answer\" size=\"8\">
</div>\n";
}

sub captcha_text1 {
    my $this = shift;
    my ($challenge,$response);
    my (@cword, $iword, $ilength, $word, @letter, $nletter, $password, $npass);
  RESTART:
    @cword = $this->get_words($config{captcha}{word_set_size});
    while (! $word) {
	$iword = int(rand(scalar @cword));
	$word = $cword[$iword];
	$ilength = length $word;
    }

  REDO_PASS: @letter = (); $password = undef; $npass=0;
    foreach (1..$ilength) {
	if (rand(2.5) < 1) {
	    push @letter,$_-1;
	    $password .= substr($word,($_-1),1);
	}
	last if (@letter == $config{captcha}{max_password_size});
    }
    if (@letter < 2) {
	$npass ++;
	#print "*** $npass $word $password\n";
	goto RESTART if ($npass > 10);
	goto REDO_PASS;
    }
    $nletter = scalar @letter;

    $challenge = "Enter the ";
    foreach (1..$nletter) {
	$challenge .= $RANK[$letter[$_-1]];
	if ($_ == $nletter - 1) { $challenge .= " and "; }
	elsif ($_ != $nletter) { $challenge .= ", "; }
    }
    $challenge .= " letters of the ".
    ($iword == $#cword ? "last" : $RANK[$iword]).
    " word in the following list:<br>\n";
    $challenge .= join(" ",@cword)."\n";

    $response = $password;
    return ($challenge,$response);
}

sub captcha_text2 {
    my $this = shift;
    my ($challenge,$response);
    my @op = ("+","-","&times;");
    my $n1 = int(rand(9))+1;
    my $n2 = int(rand(9))+1;
    my $op = $op[int(rand(scalar @op))];
    $challenge = "$n1 $op $n2 = ?";
    $response = eval("$n1$op$n2");
    return ($challenge,$response);
}

sub captcha_text3 {
    my $this = shift;
    my ($challenge,$response);
    my (@cword, $iword, $ilength, $word, @letter, $nletter, $password, $npass);

    @cword = $this->get_words($config{captcha}{word_set_size});
    $iword = int(rand(scalar @cword));
    $word = $cword[$iword];
    $ilength = length $word;

    $challenge = "How many letters are in the ".
	($iword == $#cword ? "last" : $RANK[$iword]).
	" word of the following list?<br>\n";
    $challenge .= join (" ",@cword);
    $response = $ilength;
    return ($challenge,$response);
}

sub captcha_text4 {
    my $this = shift;
    my ($challenge,$response);
    my (@cword, $iword, $ilength, $word, @letter, $nletter, $password, $npass);

    my %category = (
		    animals => [qw(DOG CAT SHARK RABBIT SNAKE BEAR WHALE ELEPHANT CROCODILE SPIDER)],
		    colours => [qw(RED BLUE YELLOW GREEN PINK PURPLE BLACK WHITE BROWN)],
		    vehicles => [qw(CAR TRUCK BOAT TRAIN SHIP AIRPLANE ROCKET BICYCLE)],
		    names => [qw(BOB MARY JOHN BILL WENDY SUSAN PATRICK JOE MARTIN FRANK TAMMY)],
		    countries => [qw(CANADA FRANCE GERMANY CHINA JAPAN BRAZIL ITALY EGYPT NORWAY)],
		    food => [qw(BANANA CHEESE BREAD BEEF POTATO SALAD PASTA CAKE CORN APPLE)],
		    );

    my @categories = keys %category;
    my ($c1,$c2);
    $c1 = $categories[int(rand(scalar @categories))];
    do { $c2 = $categories[int(rand(scalar @categories))]; } 
    until ($c1 ne $c2);
    
    my $rightword = $category{$c1}[int(rand(scalar @{$category{$c1}}))];
    my @wrongwords;
    while (@wrongwords < 4) {
	my $iword = int(rand(scalar @{$category{$c2}}));
	push @wrongwords, $category{$c2}[$iword];
	splice @{$category{$c2}},$iword,1;
    }
    my $rightpos = int(rand(scalar @wrongwords));
    splice @wrongwords,$rightpos,0,$rightword;
    
    $challenge = "Which one of these is not like the other?<br>\n".
	join(" ",@wrongwords);
    $response = $rightword;
    return ($challenge,$response);
}

sub get_words {
    my $this = shift;
    my ($nwords) = @_;
    my @word;
    open F,"<$config{captcha}{dictionary}";
    # Some dictionaries contain accented characters or words with
    # punctuation marks - strip all of these.
    @word = grep(/^[A-Za-z]+$/,<F>);
    close F;
    my $nword = scalar @word;
    my @cword = ();
    foreach (1..$nwords) {
	my $rand = int(rand($nword)) + 1;
	next if (length($word[$rand]) < 5);
	push @cword, $word[$rand];
    }
    chomp @cword;
    return @cword;
}

1;

