package ExSite::RSS;

#----------------------------------------------------------------------------
#
#   Copyright 2001-2007 Exware Solutions, Inc.  http://www.exware.com
#
#   This file is part of ExSite WebWare (ExSite, for short).
#
#   ExSite is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   ExSite is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with ExSite; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   Users requiring warranty coverage and/or support may arrange alternate
#   commercial licensing for ExSite, by contacting Exware Solutions 
#   via the website noted above.
#
#-----------------------------------------------------------------------

=pod

=head1 RSS.pm

All-purpose simple RSS feed generator.

=head2 Usage:

Create the feed (the channel descriptors are accepted without validation, 
so you must ensure they are correct):

    my $rss = new ExSite::RSS(title=>"My Feed",description=>"foo",link=>"http://myfeed.com/");

You can also specify channel descriptors separately.  This is slightly
better, because the parameters are validated and made XML-friendly.

    my $rss = new ExSite::RSS;
    $rss->title("My Feed");
    $rss->description("foo");
    $rss->link("http://myfeed.com/");

Add items to the feed.  The parameters are title, description, link.

    $rss->item("1st Title","1st description","http://myfeed.com/1");
    $rss->item("2nd Title","2nd description","http://myfeed.com/2");
    $rss->item("3rd Title","3rd description","http://myfeed.com/3");

Output the feed:

    print $rss->write;

Clear the feed so you can start over:

    $rss->clear;

=cut

use strict;
use ExSite::Config;
use ExSite::Misc;
use ExSite::Base;

use vars qw(@ISA);
@ISA = qw(ExSite::Base);

sub set {
    my ($this,$key,$val) = @_;
    $val =~ s/&/&amp;/g;
    $this->{$key} = $val;
    return;
}

sub title {
    my ($this,$val) = @_;
    $this->set("title",$val);
}

sub description {
    my ($this,$val) = @_;
    $this->set("description",$val);
}

sub link {
    my ($this,$val) = @_;
    if ($val !~ /^https?:/) { $val = $config{server}{server}.$val; }
    $this->set("link",$val);
}

sub item {
    my ($this,$title,$description,$link) = @_;
    if (! $this->{items}) { $this->{items} = []; }
    $title = $this->clean($title);
    $description = $this->clean($description);
    $link =~ s/&/&amp;/g;
    if ($link !~ /^https?:/) { $link = $config{server}{server}.$link; }
    push @{$this->{items}}, { 
	title=>$title, 
	description=>$description, 
	link=>$link 
	};
}

sub clear {
    my $this = shift;
    $this->{title} = undef;
    $this->{description} = undef;
    $this->{items} = undef;
}

sub write {
    my $this = shift;
    my $out = "<?xml version=\"1.0\"?>
<rss version=\"2.0\">
<channel>\n\n";
    if (ref $this->{items} eq "ARRAY" && 
	@{$this->{items}} > 0) {
	$out .= "<title>$this->{title}</title>
<description>$this->{description}</description>
<link>$this->{link}</link>\n";
 
	foreach my $item (@{$this->{items}}) {
	    $out .= "
<item>
<title>$item->{title}</title>
<description>$item->{description}</description>
<link>$item->{link}</link>
</item>\n";
	}
    }
    else {
	$this->warn("no items in RSS feed");
    }
    $out .= "\n</channel>\n</rss>\n";
    return $out;
}

# Clean up RSS text for display: Some browsers (okay, just IE) are 
# intolerant of non-ASCII characters if you have not specified an encoding 
# type in the XML.  Unfortunately, it's hard for us to say with certainty 
# what the encoding type is, since the content may come from a variety of
# sources.  The work-around is to assume a Latin-1 character set, and
# convert it to an ASCII facsimile of the original content.  This works
# for most western, non-ASCII content (eg. French) in an ISO-8859 or 
# similar encoding, but does not work well for UTF-8 encoding on 
# non-ASCII Latin character sets, and does not work at all for non-Latin 
# character sets.

sub clean {
    my ($this,$text) = @_;
    $text =~ s/&/&amp;/g;
    return &latin1_to_ascii($text);
}

1;
