package ExSite::Search;

#----------------------------------------------------------------------------
#
#   Copyright 2001-2007 Exware Solutions, Inc.  http://www.exware.com
#
#   This file is part of ExSite WebWare (ExSite, for short).
#
#   ExSite is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   ExSite is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with ExSite; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   Users requiring warranty coverage and/or support may arrange alternate
#   commercial licensing for ExSite, by contacting Exware Solutions 
#   via the website noted above.
#
#-----------------------------------------------------------------------

=pod

=head1 Search.pm

The search system is based on a simple search index that uses 2 tables:

=over 4

=item * searchurl

every URL that is searched is represented once here

=item * searchterm

every indexed word at each url is represented once here,
with a weight (based on location and number of ocurrences).

=back

The search index is constructed by finding all content that is unique
to a URL (eg. the content objects that are page-specific), stripping
out all mark-up, and counting the incidence of each remaining word in
the text.  For web pages, we also index words in filenames, titles,
descriptions, and keywords.

Indexing of a web site consists of:

=over 4

=item 1 indexing the regular content on each page

The search system finds this content and indexes it automatically.  It
ignores content that is not page-specific, such as text that comes
from templates, menus, and so forth.

=item 2 indexing the content in each plug-in

Individual plug-ins can devise their own content indexing logic.  To
make a plug-in search indexing tool available to the system, the
plugin must be defined as a site service, and must reply with a code
reference to the the "Search" ioctl command.  This code reference is
the plugin's search indexer.  It will be invoked with three
parameters:

=over 4

=item an ExSite::Search object

Using this, the plug-in can add terms into the search index.

=item an ExSite::Section object

Using this, the plug-in can constrain which section's content gets indexed.

=item an ExSite::Page object

This indicates the page that should be used to deliver the plugin's
content in saerch results.  In other words, the plug-in should index
URLs that generate this page, albeit with alternate query string
parameters.

=back

=back

=head2 Bugs & Limitations

The search logic is somewhat crude, and does not allow for searching
for partial words, quoted phrases, absence of words, etc.

It does not index alt tags on images.

It does not index any plug-ins that have not been configured as a service.

=cut

use strict;
use ExSite::Config;
use ExSite::Auth;
use ExSite::Section;
use ExSite::Module qw(&get_module);
use ExSite::Page;
use ExSite::Base;

use vars qw(@ISA);
@ISA = qw(ExSite::Base);

sub new {
    my $this = shift;
    my $obj = {};
    $obj->{section_id} = shift;
    my $class = ref($this) || $this;
    bless $obj, $class;
    $obj->initialize_object;

    $obj->{weight} = $config{search}{weight};
    my @skipwords = split / /,$config{search}{skipwords};
    $obj->{skipwords} = \@skipwords;

    $obj->reset;

    return $obj;
}

sub reset {
    my $this = shift;
    $this->{title} = {};
    $this->{descr} = {};
    $this->{site} = {};
    $this->{privacy} = {};
    $this->{clear_section} = [];
    $this->{clear_url} = [];
    return;
}

sub search_form {
    my ($this,$term,$title,$width) = @_;
    my $url = $this->link(search=>"1");
    if (! $width) { $width = 20; }
    my $out;
    if ($title) { $out .= "<h1>$title</h1>"; }
    $out .= "\n<form method=\"POST\" action=\"$url\" class=\"search\" id=\"searchform\"><input type=\"text\" size=\"$width\" name=\"searchterm\" value=\"$term\"><input type=\"submit\" value=\"Search\"></form>\n";
    return $out;
}

sub do_search {
    my ($this,$term) = @_;
    my $out .= "<h1>Search Results for \"$term\"</h1>\n";
    $out .= $this->display_results($this->search($term));
    $out .= $this->search_form($term,"Search");
    return $out;
}

# index_site : indexes all content in a site

sub index_site {
    my ($this,$section) = @_;
    my $site = ref $section ?
	new ExSite::Section(section=>$section) :
	new ExSite::Section(id=>$section);

    # first, index all of the site's pages
    my $out .= $this->index_pages($site->get_all_pages(status=>"active"));

    # second, see if any of the website services wish to add to the index
    $out .= $this->index_plugins($site);

    # remember to clear old index for this site before updating
    push @{$this->{clear_section}},$site->id();

    return $out;
}

# index_plugins : indexes special content for all plugins in a site

sub index_plugins {
    my ($this,$section) = @_;
    my ($site,$out);
    if ((ref $section) =~ /Section/) {
	$site = $section;
    } 
    else {
	if (ref $section) { $site = new ExSite::Section(section=>$section); }
	else { $site = new ExSite::Section(id=>$section); }
    }
    my @service = $site->get_services;
    foreach my $service (@service) {
	if ($service->{page_id}) {
	    my $p = new ExSite::Page(id=>$service->{page_id});
	    my $modname = $service->{name};
	    my $dcd = &get_module($modname,0,$section);
	    #my $dcd = $site->get_dcd($modname);
	    if ($dcd) {
		$out .= $this->index_plugin($dcd,$site,$p);
	    }
	}
    }
    return $out;
}

# index_plugin : indexes special content for a specific plugin

sub index_plugin {
    my ($this,$dcd,$section,$page) = @_;
# this fails for reasons unknown
#    my $search = eval "$dcd->ioctl('Search')";
#    if ($@) {
#	return $this->error("Search::index_plugin: $@");
#    }
    my $search = $dcd->ioctl('Search');
    if ($search) {
	# this service has its own search indexing tools
	return $dcd->$search($this,$section,$page);
    }
}

sub index_pages {
    my ($this,@pages) = @_;
    my $out;
    foreach my $p (@pages) {
	$out .= $this->index_page($p);
	if ($config{search}{scanpause}) { sleep($config{search}{scanpause}); }
    }
    return $out;
}

# indexpage : index the metadata and content of a page

sub index_page {
    my ($this,$page) = @_;
    if (ref $page !~ /Page/) {
	if (ref $page) { $page = new ExSite::Page(page=>$page); }
	else { $page = new ExSite::Page(id=>$page); }
    }
    return undef if (! $page->is_page);   # do not index libs, templates
    #return undef if (! $page->is_public); # do not index private pages
    $share{Page} = $page;
    $page->set_context;
    my $url = $page->get_url;
    return $this->error("$url: Permission denied") 
	if (! $share{DB}->owns($page->get_my("section_id")));
    my $filename = $page->filename;
    my $title = $page->title;
    my $descr = $page->description;
    my $privacy = $page->is_public ? "public" : "members-only";
    $this->add_url($url,$page->id("section"),$title,$descr,$privacy);
    
    my @content = $page->fetch_all_content_objs();
    foreach my $c (@content) {
	$c->set_context();
	my $ctype = $c->content_type;
	next if ($ctype !~ /^text/);
	my $text = $c->get_raw;
	if ($ctype =~ /html/) {
	    $c->expand(html=>$text);
	    $text = $c->{html};
	    # remove stuff outside the body
	    if ($text =~ /<body>(.+)<\/body>/si) { $text = $1; }
	    # remove CSS
	    $text =~ s/<style>.+?<\/style>/ /gsi;
	    # remove Javascript
	    $text =~ s/<script.*?>.+?<\/script>/ /gsi;
	    # remove HTML tags
	    $text =~ s/<\/?[^>]+>/ /g;
	}
	$this->index($url,$this->{weight}{body},$text);
    }

    $this->index($url,$this->{weight}{title},$title);
    $this->index($url,$this->{weight}{keywords},$page->keywords);
    $this->index($url,$this->{weight}{description},$descr);
#    $this->index($url,$this->{weight}{body},$body);
    $this->index($url,$this->{weight}{filename},$filename);

    return "indexing $url...<br>\n";
}

# index : indexes the words in a block of plaintext

sub index {
    my ($this,$url,$weight,$text) = @_;
    return if (!$text);
    return if (! exists $this->{index}{$url});

    # strip endlines
    $text =~ s/\r?\n/ /g;
    # strip all non-word characters
    $text =~ s/[^\w]/ /g;
    # convert to lower case
    $text = lc($text);
    # convert to an array of words
    my @word = split /\s+/,$text;

    # index the words found
    foreach my $word (@word) {
	$this->index_term($url,$word,$weight);
    }
}

# index_term : add a term to the index

sub index_term {
    my ($this,$url,$term,$weight) = @_;
    return 0 if (! $url);
    return 0 if (! $term);
    return 0 if (grep /^$term$/,@{$this->{skipwords}});
    $weight |= 1;
    $this->{index}{$url}{$term} += $weight;
    # should return current weight for that term
}

# setup_url : add a url to the index

sub add_url {
    my ($this,$url,$section,$title,$descr,$privacy) = @_;
    if (! exists $this->{index}{$url}) {
	$this->{index}{$url} = {};
	# remember to clear old index for this URL before updating
	push @{$this->{clear_url}},$url;
    }
    $this->{title}{$url} = $title;
    $this->{descr}{$url} = $descr;
    $this->{site}{$url} = $section;
    $this->{privacy}{$url} = $privacy || "public";
    return;
}

# clear : clear old search data for a particular site

sub clear_site {
    my ($this,$section_id) = @_;
    my $db = $share{DB};
    if ($section_id) {
	my @oldurl = $db->fetch_match("searchurl",{section_id=>$section_id});
	foreach my $oldurl (@oldurl) {
	    # delete links and urls
	    $this->clear_url($oldurl->{searchurl_id});
	}
    }
}

# clear search data for a single URL
# $url can be a searchurl_id or a full URL

sub clear_url {
    my ($this,$url) = @_;
    my $db = $share{DB};
    my @urlid;
    if ($url =~ /^\d+$/) { 
	$db->delete("searchterm",{searchurl_id=>$url});
	$db->delete("searchurl",{searchurl_id=>$url});
    }
    else {
	my @oldurl = $db->fetch_match("searchurl",{url=>$url});
	foreach my $u (@oldurl) {
	    if ($db->owns($u->{section_id})) {
		$db->delete("searchterm",{searchurl_id=>$u->{searchurl_id}});
		$db->delete("searchurl",{searchurl_id=>$u->{searchurl_id}});
	    }
	}
    }
}

# update : update the DB index

sub update {
    my ($this,$threshold) = @_;
    if (!$threshold) { $threshold = 1; }
    my $db = $share{DB};

    # first, remove older indexes
    foreach my $site (@{$this->{clear_section}}) {
	$this->clear_site($site);
    }
    foreach my $url (@{$this->{clear_url}}) {
	$this->clear_url($url);
    }

    # then, add the new indexing data
    foreach my $url (keys %{$this->{index}}) {

	# clear old search data
	my @oldurl = $db->fetch_match("searchurl",{url=>$url});
	foreach my $oldurl (@oldurl) {
	    $this->clear_url($oldurl->{searchurl_id});
	}
	
	# add new data
	my $url_id = $db->insert("searchurl",{url=>$url,
					      title=>$this->{title}{$url},
					      description=>$this->{descr}{$url},
					      section_id=>$this->{site}{$url},
					      privacy=>$this->{privacy}{$url},
					      });
	foreach my $word (keys %{$this->{index}{$url}}) {
	    next if (! $word);
	    $db->insert("searchterm",{searchurl_id=>$url_id,
				      term=>$word,
				      weight=>$this->{index}{$url}{$word}});
	}
    }
}

# search the search table

sub search {
    my ($this,$terms) = @_;

    my $db = $share{DB};
    $terms =~ s/[^\w]/ /g;  # eliminate punctuation
    my @term = split /\s+/,lc($terms);
    my $nterm = scalar @term;

    # FIXME: this search finds results across all sites in the system.
    # Hits pertaining to the current site are filtered in display_results,
    # but if the number of unrelated sites is large, this will become a
    # drag on memory and performance.  In that case the extra hit of a join 
    # with searchurl may be more appropriate.

    my @termresults;
    foreach my $term (@term) {
	push @termresults, $db->fetch_match("searchterm",{term=>$term});
    }

    # combine the results into a single result hash

    my (@url,%result);
    foreach my $termresult (@termresults) {
	$result{$termresult->{searchurl_id}} += ($termresult->{weight} / $nterm);
    }
    
    return %result;
}

# format the search results 

sub display_results {
    my ($this,%results) = @_;
    my @url_id = sort { $results{$b} <=> $results{$a}; } keys %results;
    my $out = "<dl>\n";
    my $db = $share{DB};
    my $nresult = 0;
    # only display results from this section
    my $section = $share{Page}->id("section");
    foreach my $url_id (@url_id) {
	my $url = $db->fetch("searchurl",$url_id);
	next if ($url->{section_id} != $section);  # wrong site
	next if ($url->{privacy} && 
		 $url->{privacy} ne "public" && 
		 ! $db->level); # members only
	$nresult++;
	my $score = int($results{$url_id}*10)/10;
	if ($url->{url} =~ /^$config{server}{CGIpath}/) {
	    # show complete URLs
	    $url->{url} = $config{server}{server}.$url->{url};
	}
	$out .= "<dt><a href=\"$url->{url}\">$url->{title}</a></dt>\n";
	$out .= "<dd>$url->{description}";
	if ($url->{description}) { $out .= "<br>\n"; }
	$out .= "<font size=1>[$url->{url}, score: $score]</font><p>\n";
	last if ($nresult == $config{search}{maxresult});
    }
    if ($nresult == 0) {
	$out .= "<dt>Nothing relevant was found.</dt>\n";
    }
    $out .= "</dl>\n";
}

1;
