#-----------------------------------------------------------------------
#
#   Copyright 2001-2006 Exware Solutions, Inc.  http://www.exware.com
#
#   This file is part of ExSite WebWare (ExSite, for short).
#
#   ExSite is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   ExSite is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with ExSite; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   Users requiring warranty coverage and/or support may arrange alternate
#   commercial licensing for ExSite, by contacting Exware Solutions 
#   via the website noted above.
#
#----------------------------------------------------------------------------
=pod

=head1 ExSite::StoreDBM

Implements the ExSite::Store class on a DBM storage engine.

Note that Store engines can only store scalars or equivalent.  That means
that complex data items must be frozen using Storable before being passed
to us here.  That work is handled for us by the ExSite::Store wrapper.

=cut

package ExSite::StoreDBM;

use strict;
use Fcntl qw(:flock);

# inherit from the base store
use ExSite::StoreBase;
use vars qw(@ISA);
@ISA = qw(ExSite::StoreBase);

#------------------------------------------------------------------------
# Public methods are accessed indirectly using using perl's tie mechanism.
#
# NB: most tie methods are inherited from StoreBase
#------------------------------------------------------------------------

sub STORE {
    my ($this,$key,$val) = @_;
    if ($this->lock) {
	# ok to proceed
	my $data = $this->encode($val);
	$this->{store}{$key} = $data or $this->log("store $key failed",$@);
#	if (eval {$this->{store}{$key} = $data;} != $data) {
	    # value did not save 
	    
#	}
    }
    $this->unlock;
    # fail silently if no file lock
    return $val;
}

sub EXISTS {
    my ($this,$key) = @_;
    # Any_DBM doesn't support exists, so we simply check for a defined value
    my $dat = $this->{store}{$key};
    return (defined $dat);
}

#------------------------------------------------------------------------
# Internal methods (not accessed via the tie interface).
#------------------------------------------------------------------------

# connect:  connect to the DB only if it exists

sub connect {
    my ($this) = @_;
    # test for lock file, since DBM files can have different suffixes
    if (-e $this->{config}{DBM_lockfilename}) {
	return $this->make;
    }
    return undef;
}

sub make {
    my ($this) = @_;

    my %store;

    # if the files do not exist, the caller will need sufficient 
    # privileges to create them

    if (dbmopen(%store,$this->{config}{DBM_filename},0666)) { 
	$this->{store} = \%store;
	if (! -e $this->{config}{DBM_lockfilename}) {
	    if (open F, ">$this->{config}{DBM_lockfilename}") {
		close F;
		if (! chmod 0666, $this->{config}{DBM_lockfilename}) {
		    $this->log($this->{config}{DBM_lockfilename},$!);
		    return 0;
		}
	    }
	    else {
		$this->log($this->{config}{DBM_lockfilename},$!);
		return 0;
	    }
	}
	return 1;
    }
    else {
	# no store table
	$this->log("error opening StoreDBM table");
	dbmclose(%store);  # probably unnecessary
	return undef;
    }
}

sub remove {
    my ($this) = @_;
    if (-e $this->{config}{DBM_filename}) {
	unlink $this->{config}{DBM_filename};
    }
    if (-e $this->{config}{DBM_lockfilename}) {
	unlink $this->{config}{DBM_lockfilename};
    }
}

# multistore: save multiple key/value pairs at once

sub multistore {
    my ($this,%data) = @_;
    if ($this->lock) {
	# we lock only once for efficiency
	while (my ($key,$val) = each %data) {
	    $this->{store}{$key} = $val;
	}
	$this->unlock;
    }
    # fail silently if bad file lock
}

#------------------------------------------------------------------------
# Locking tools, to prevent file collisions.
#------------------------------------------------------------------------
#
# Lock/unlock a DB for read-write access
# -> doesn't lock DB itself;  uses a lockfile for simplicity
#    will fail if lockfile is missing, or not writeable.
# -> returns TRUE (1) if lock was successful
# -> calling script must police itself - a misbehaving script
#    could ignore the lock and proceed anyway.
#
#------------------------------------------------------------------------

sub lock {
    my $this = shift;
    # first, test the lock file for write access
    if (open LOCK,$this->{config}{DBM_lockfilename}) {
	if (! flock(LOCK,LOCK_EX)) {
	    $this->log("failed to lock store lockfile");
	    return 0;
	}
    }
    else {
	$this->log("failed to open store lockfile");
	return 0;
    }
    return 1;
}

# non-blocking version - aborts after 10sec timeout
sub lock_nb {
    my $this = shift;
    # first, test the lock file for write access
    if (open LOCK,$this->{config}{DBM_lockfilename}) {
	my $count = 0;
	my $gotit = flock LOCK,(LOCK_EX|LOCK_NB);
	while (! $gotit && $count < 10) {
	    $count++;
	    sleep(1);
	    $gotit = flock LOCK,(LOCK_EX|LOCK_NB);
	}
        if (! $gotit) { 
	    $this->log("session lockfile timed out");
	    return 0; 
	}
    }
    else { 
	$this->log("failed to open session lockfile");
	return 0; 
    }
    return 1;
}

sub unlock {
    flock LOCK,LOCK_UN;
    close LOCK;
}

1;
