#!/usr/bin/perl
#----------------------------------------------------------------------------
#
#   Copyright 2001-2004 Exware Solutions, Inc.  http://www.exware.com
#
#   This file is part of ExSite WebWare (ExSite, for short).
#
#   ExSite is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   ExSite is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with ExSite; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   Users requiring warranty coverage and/or support may arrange alternate
#   commercial licensing for ExSite, by contacting Exware Solutions 
#   via the website noted above.
#
#----------------------------------------------------------------------------

package ExSite::Util;

use strict;
use ExSite::Misc;
use ExSite::Config;

#========================================================================

=pod

=head1 ExSite::Util --  miscellaneous utility functions needed by ExSite::

Miscellaneous routines and functions that depend on ExSite::Config only

=cut

#========================================================================

BEGIN {
    use Exporter   ();
    use vars       qw($VERSION @ISA @EXPORT);
    $VERSION     = 1.00;
    @ISA         = qw(Exporter);
    @EXPORT = qw(   &replace_tag &restore_cms_tags 
		    &restore_cms_tag_img &restore_cms_tag_dcd
		    &set_cookie &get_cookie &edit_cookie 
		    &redirect &redirect_http &redirect_html
		    &snippet
		    &run_publisher
		    &parse_url
		    );
}

### Cookie-handling routines are deprecated; use ExSite::Cookie instead.

sub set_cookie {
    my ($name,$value,$path,$domain,$expiry) = @_;
    if (! $path) { $path = $config{server}->{CGIpath}; }
    if (! $domain) { 
	$domain = $config{server}{host} ? 
	    ".$config{server}{domain}" : $config{server}{domain}; 
    }
    my $out = "Set-cookie: $name=$value; path=$path; domain=$domain";
    if ($expiry) { $out .= "; Expires=$expiry"; }
    return "$out\n";
}

sub get_cookie { 
    my ($name,$sepchar) = @_;
    if (exists $ENV{'HTTP_COOKIE'}) {
        my @cookies = split /; /, $ENV{'HTTP_COOKIE'};
        foreach my $cookie (@cookies) {
            if ($cookie =~ /^$name=/) {
		my (undef,$value) = split /=/,$cookie,2;
		if ($sepchar) { 
		    return split /$sepchar/,$value;
		}
		else {
		    return $value;
		}
            }
        }
    }
    return undef;
}

sub edit_cookie { 
    my ($name,$value) = @_;
    my @cookies;
    my $done;
    if (exists $ENV{'HTTP_COOKIE'}) {
        @cookies = split /; /, $ENV{'HTTP_COOKIE'};
        foreach my $cookie (@cookies) {
            if ($cookie =~ /^$name=/) {
		$cookie = "$name=$value";
		$done = 1;
		last;
	    }
        }
    }
    if (! $done) {
	# add the cookie
	push @cookies, "$name=$value";
    }
    # redefine the cookie
    my $new_cookie = join "; ",@cookies;
    $ENV{HTTP_COOKIE} = $new_cookie;
    return $new_cookie;
}

=pod

=head2 redirect($goto) - change to a different URL.

This is presently an alias for C<redirect_http()>, which switches URLs 
using a C<Location:> header.  Partial (relative) URLs may leave the
address bar indicating the previous URLs, although the content will
reflect the new URL.

As an alternative, you can use C<redirect_html()>, which switches URLs
using a meta-refresh HTML tag.  This should update the address bar no matter
what URL you pass.  It also outputs a brief redirection message, which 
also allows the user to click a link to redirect in case the redirection
fails or is not supported by the browser.  However, it results in two 
distinct pages being served, which is slower.

=cut

sub redirect {
    &redirect_http(@_);
}

sub redirect_http {
    my $goto = shift;

    # look for page management objects that will handle redirection 
    # gracefully, even if we have already generated output.

    my $p = $share{Page} || $share{PageBuf};
    if ($p) { 
	$p->location($goto);
    }
    else {
	# no page management objects found
	# try printing the location header directly
	print "Location: $goto\n";
    }
    return;
}

sub redirect_html {
    my $goto = shift;

    # look for page management objects that may contain buffered output;
    # cancel this output

    my $p = $share{Page} || $share{PageBuf};
    if ($p) { 
	$p->cancel();
    }

    # now output a dummy page that contains a meta-refresh header

    print <<EOF;
Content-type:text/html

<html>
<head>
<META HTTP-EQUIV="Refresh" CONTENT="0;URL=$goto">
</head>
<body>
Redirecting to <a href="$goto">$goto</a>...
</body>
</html>
EOF
    return;
}

=pod

=head2 snippet()

generates preformatted HTML snippets with data insertions

Snippets are stored in C<$config{snippet}-E<GT>{snippet-name}>.  Each
snippet is an array of values, that are simply copied to the output
string in order, subject to the following transformations:

=over 4

=item * strings: sent to output as-is

=item * numbers: sends the calling argument at that index to the output.

=item * code reference: executes that code, and sends the return value
to the ouput.

=back

=cut

sub snippet {
    my $snippet = $_[0];
    my $out;
    foreach my $dat (@{$config{snippet}->{$snippet}}) { 
	if (ref $dat eq "CODE") { $out .= &$dat; }
	elsif ($dat =~ /^\d$/) { $out .= $_[$dat]; }
	else { $out .= $dat; }
    }
    return $out;
}


=pod

=head1 CMS tag utilities

=head2 replace_tag($html)

replaces the CMS tags in $html with IMG tags that serve as
placeholders, and allow the HTML to be displayed out of context.
DEPRECATED: ExSite::ContentBase::expand() dummy modes now do this
task.

=head2 restore_cms_tags($html)

convert masked CMS tags (see replace_tag) back to the original CMS
tags.

This routine contains a lot of ugly kludges to clean up after some 
browsers that alter our HTML for reasons that probably seemed like a good
idea to the browser authors, but which have obligated us to call down a 
plague of locusts upon them for messing with our stuff.

For example:

=over 4

=item IE randomly rearranges the attributes in our IMG tags, making it hard 
to find the ones we just inserted into the code.

=item IE rewrites our HREF URLs to add hostname info, thereby trashing our
page pointer tags.

=item several browsers like to convert '&' in URLs to '&amp;' and then 
convert it back again so that scripts etc. don't notice.  But *we* notice...

=back

=cut


sub replace_tag
{
    my ($html) = @_;

    # replace <img src=[[...]]> tags

    $html =~ s/\<img src=(\'|\")?\[\[(\w+\.\w+)\]\](\'|\")?[^>]*\>/\<img alt=\"\[\[$2\]\]\" name=\"$2\" src=\"$config{server}->{HTMLpath}\/_ExSite\/images\/checker.png\" title=\"$2\" width=\"100\" height=\"50\"\>/gi;

    # replace <!--&content(...)--> tags

    $html =~ s/\<!--\&?content\((\w+)\)--\>/\<img alt=\"content($1)\" name=\"$1\" src=\"$config{server}->{HTMLpath}\/_ExSite\/images\/checker.png\" title=\"$1\" width=\"100\" height=\"50\"\>/gi;

    # replace DCD tags <!--&XYZ(...)-->

    $html =~ s/\<!--\&(\w+)\((\w*)\)--\>/\<img alt=\"$1($2)\" name=\"$1\" src=\"$config{server}->{HTMLpath}\/_ExSite\/images\/checker.png\" title=\"$1\" width=\"100\" height=\"50\"\>/gi;

    return $html;
}

sub restore_cms_tags {
    my ($data) = @_;

    # restore <!--&content(...)--> tags
    $data =~ s/\<img ([^>]*)alt=\"?content\((.*?)\)\"?.*?\>/\<!--content($2)--\>/gi;

    # restore masked URL substitutions - [|[name]|]
    $data =~ s/\[\|\[(.*?)\]\|\]/[[$1]]/g;

    # restore dummy URL-substituted graphics (deprecated?)
    $data =~ s/(<img ([^>]*)alt=\"?\[\[([-\w]+\.?\w+)\]\]\"?[^>]*\>)/&restore_cms_tag_img($1)/gie;

    # restore real URL-substituted graphcs
    $data =~ s/($config{server}{server})?$config{server}{CGIpath}\/$config{prog}{content}(\/)?(.+?)\?id=(\d+)&(amp;)?name=([\w-\%\.]+)/[[$6]]/gi;

    # restore content tags <!--content(name)-->
    $data =~ s/\<img ([^>]*)alt=\"?content\(.*?\)\"?[^>]*\>/<!--content($2)-->/gie;

    # restore dummy DCD objects
    $data =~ s/\<img ([^>]*)alt=\"?((&(amp;)?)*\w+)\(([^\"]*)\)\"?[^>]*\>/&restore_cms_tag_dcd($2,$5)/gie;

    # KLUDGE: unescape URL-substitution tags that are mysteriously escaped 
    # by some browsers
    $data =~ s/%7B%7B/{{/g;
    $data =~ s/%7D%7D/}}/g;
    $data =~ s/%5B%5B/\[\[/g;
    $data =~ s/%5D%5D/\]\]/g;

    # KLUDGE: fix page-URL substitutions that get trashed by IE
    # IE wants to convert '<a href={{x}}>' 
    #                  to '<a href="http://whatever.com/cgi/{{x}}>'
    # so we have to strip out the gak that it has so helpfully inserted
    # for us.
    $data =~ s/href=\"[^{>]+({{[^}]+}})\"/href=\"$1\"/g;
    $data =~ s/href=\"[^\[>]+(\[\[[^\]]+\]\])\"/href=\"$1\"/g;

    return $data;
}

# restoring images is complicated, because IE scrambles the attributes, 
# which means we cannot match to reliable patterns.

sub restore_cms_tag_img {
    my $tag = shift;
    my ($name,$align,$border);
    # 041116 - allow hyphens in content names
    # 050109 - allow escape chars (%) in content names
    if ($tag =~ /alt=\"?\[\[([-%\w]+\.?\w+)\]\]/i) { $name = $1; }
    my $newtag = "<img src=\"[[$name]]\"";
    if ($tag =~ /align=\"?(\w+)\"?[ >]/i) { 
	$align = $1; 
	$newtag .= " align=\"$align\"";
    }
    if ($tag =~ /border=\"?(\d+)\"?[ >]/i) { 
	$border = $1; 
	$newtag .= " border=\"$border\"";
    }
    $newtag .= ">";
    return $newtag;
}

# DCD tags get the parameter separator '&' munged by browsers. We should
# switch to using ';' instead, but in the meantime, this kludge un-munges 
# the '&' characters in the tags.

sub restore_cms_tag_dcd {
    my ($dcd,$param) = @_;
    $dcd =~ s/\&amp\;/\&/g; # dcd name can have embedded & chars, if using Ajax
    $param =~ s/\&amp\;/\&/g;
    return "<!--\&$dcd($param)-->";
}

#------------------------------------------------------------------------
# Unpriveleged publishing - if we need to publish files from a URL other
# than publish.cgi, we have to do so by running publish.cgi, since no other
# programs have sufficient priveleges.
#
# Admin privileges are still required to do this.  To run this as an 
# unprivileged user, you must pass an authtoken in the %$other args.
#
# $what - section|page|content|module
# $id - numeric id, or module name
# $unpublish - true (1) if we are removing, rather than creating, files
# $other - hash ref to other publish args

sub run_publisher {
    my ($what,$id,$unpublish,$other) = @_;

    my %args = (ref $other eq "HASH") ? %$other : ();
    my $whatkey = $unpublish ? "-".$what : $what;
    $args{$whatkey} = $id;

    # overload environment
    $ENV{QUERY_STRING} = &EncodeHash(%args);
    $ENV{SCRIPT_NAME} = "$config{server}{CGIpath}/$config{prog}{publish}";
    $ENV{REQUEST_METHOD} = "GET";

    my $out = `$config{server}{CGIroot}/$config{prog}{publish}`;

    # restore the environment of the current request, just to be safe
    $ENV{QUERY_STRING} = $share{ENV}{QUERY_STRING};
    $ENV{SCRIPT_NAME} = $share{ENV}{SCRIPT_NAME};
    $ENV{REQUEST_METHOD} = $share{ENV}{REQUEST_METHOD};

    return $out || $?;
}

# parse a URL into its component parts
# $protocol://$host$scriptpath/$scriptfile$pathinfo?$query
#
# can optionally pass in a regexp to identify scripts;
# by default, we use $config{prog} to recognize scripts.

sub parse_url {
    my $url = shift;
    my $scriptre = shift || join "|",values %{$config{prog}};
    my ($script,$query) = split /\?/, $url, 2;
    my ($protocol,$hostpath) = split /\:\/\//, $script;
    my (@path,$host);
    if ($hostpath) { 
	@path = split /\//,$hostpath;
	$host = shift @path;
    }
    else {
	# missing protocol - partial url?  All our data is probably in protocol
	@path = split /\//,$protocol;
	$protocol = $config{server}{protocol};
	$host = "$config{server}{host}.$config{server}{domain}";
    }
    my $scriptfile = undef;
    my (@scriptpath,@pathinfo);
    while (@path) {
	my $pathel = shift @path;
	if ($pathel =~ /$scriptre/) { $scriptfile = $pathel; }
	elsif ($scriptfile) { push @pathinfo,$pathel; }
	else { push @scriptpath,$pathel; }
    }
    my $scriptpath = join("/",@scriptpath);
    my $pathinfo = join("/",undef,@pathinfo);
    return ($protocol,$host,$scriptpath,$scriptfile,$pathinfo,$query);
}

=pod

Here is an alternative URL-parsing approach from the W3C docs:

Berners-Lee, et al.         Standards Track                    [Page 50]
RFC 3986                   URI Generic Syntax               January 2005

   As the "first-match-wins" algorithm is identical to the "greedy"
   disambiguation method used by POSIX regular expressions, it is
   natural and commonplace to use a regular expression for parsing the
   potential five components of a URI reference.

   The following line is the regular expression for breaking-down a
   well-formed URI reference into its components.

      ^(([^:/?#]+):)?(//([^/?#]*))?([^?#]*)(\?([^#]*))?(#(.*))?
       12            3  4          5       6  7        8 9

   The numbers in the second line above are only to assist readability;
   they indicate the reference points for each subexpression (i.e., each
   paired parenthesis).  We refer to the value matched for subexpression
   <n> as $<n>.  For example, matching the above expression to

      http://www.ics.uci.edu/pub/ietf/uri/#Related

   results in the following subexpression matches:

      $1 = http:
      $2 = http
      $3 = //www.ics.uci.edu
      $4 = www.ics.uci.edu
      $5 = /pub/ietf/uri/
      $6 = <undefined>
      $7 = <undefined>
      $8 = #Related
      $9 = Related

   where <undefined> indicates that the component is not present, as is
   the case for the query component in the above example.  Therefore, we
   can determine the value of the five components as

      scheme    = $2
      authority = $4
      path      = $5
      query     = $7
      fragment  = $9

=cut


1;
